import maze
import mazeIO
import maze_solver_skeleton as maze_solver
import maze_creator_skeleton as maze_creator
import unittest
import copy
import random

class TestMaze(unittest.TestCase):
    
    def __runIsPathTest(self, maze, path, expanded):
        #check that the path starts at the beginning and ends at
        #the end and goes through only open areas
        self.assertTrue(len(path) > 0)
        self.assertEquals(path[0], maze.start)
        self.assertEquals(path[len(path)-1], maze.end)
        for i in range(1, len(path)-1):
            self.assertEquals(maze.matrix[path[i][0]][path[i][1]], 'O')
        #check that we only visited open nodes
        for point in expanded:
            self.assertTrue(maze.matrix[point[0]][point[1]] == 'E' or \
                            maze.matrix[point[0]][point[1]] == 'B' or \
                            maze.matrix[point[0]][point[1]] == 'O')

    def __testmaze(self, search):
        maze = mazeIO.ppmIO.read_maze_from_ppm("test_maze.ppm", 6)
        solver = maze_solver.MazeSolver(copy.deepcopy(maze))
        path, expanded = solver.solve_maze(search)
        self.__runIsPathTest(maze, path, expanded)
        self.assertEquals(len(path), 361)

    def test1DFS(self):
        self.__testmaze("DFS")

    def test2BFS(self):
        self.__testmaze("BFS")
    

    def test3MazeCreation(self):
        #test a couple different maze creations
        creator = maze_creator.MazeCreator(100, 200)
        print "\n"
        for i in range(10):
            print "Checking maze ", i, " of 10"
            startrow = random.randint(1, 98)
            startcol = random.randint(1, 98)
            maze = creator.create_maze([startrow, startcol])
            solver = maze_solver.MazeSolver(copy.deepcopy(maze))
            path, expanded = solver.solve_maze("DFS")
            self.__runIsPathTest(maze, path, expanded)

    def test4BFSvsDFS(self):
        maze = mazeIO.asciiIO.read_maze_from_ascii("many_paths.txt")
        bfssolver = maze_solver.MazeSolver(copy.deepcopy(maze))
        bfspath, bfsexpanded = bfssolver.solve_maze("BFS")
        dfssolver = maze_solver.MazeSolver(copy.deepcopy(maze))
        dfspath, dfsexpanded = dfssolver.solve_maze("DFS")
        self.assertTrue(len(dfspath) >= len(bfspath))
                        
if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(TestMaze)
    unittest.TextTestRunner(verbosity=2).run(suite)
